--[[ This is generated code! See src/scripts/prefabs/clockwork_trap_teeth.moon for the actual source. ]]--
local MYTUNING = TUNING.CLOCKWORK_TRAP_TEETH
local Fueled = require("components/fueled")
local assets = {
  Asset("ANIM", "anim/clockwork_trap_teeth.zip")
}
local OnExplode
OnExplode = function(self, target)
  self.AnimState:PlayAnimation("trap")
  if target then
    self.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_trigger")
    target.components.combat:GetAttacked(self, MYTUNING.DAMAGE)
  end
  if self.components.fueled then
    self.components.fueled:DoDelta(-1)
  end
  return self:PushEvent("ontrigger", {
    target = target
  })
end
local SetSprung
SetSprung = function(self)
  return self.AnimState:PlayAnimation("trap_idle")
end
local SetInactive
SetInactive = function(self)
  return self.AnimState:PlayAnimation("inactive")
end
local CanReset
CanReset = function(self)
  local fueled
  fueled = self.components.fueled
  return not (fueled and fueled:IsEmpty())
end
local DoReset
DoReset = function(self)
  local mine
  mine = self.components.mine
  if self:IsValid() and mine then
    if CanReset(self) then
      return mine:Reset()
    else
      mine.inactive = false
      mine.issprung = true
      return SetSprung(self)
    end
  end
end
local CancelDelayedReset
CancelDelayedReset = function(self)
  if self.task then
    self.task:Cancel()
    self.task = nil
  end
  self.reset_targettime = nil
end
local StartDelayedReset
StartDelayedReset = function(self, delay)
  CancelDelayedReset(self)
  if CanReset(self) then
    delay = math.max(0, (delay or MYTUNING.RESET_DELAY))
    self.reset_targettime = GetTime() + delay
    self.task = self:DoTaskInTime(delay, DoReset)
  end
end
local OnReset
OnReset = function(self)
  self.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_reset")
  self.AnimState:PlayAnimation("reset")
  self.AnimState:PushAnimation("idle", false)
  return CancelDelayedReset(self)
end
local TryStartDelayedReset
TryStartDelayedReset = function(self)
  local mine
  mine = self.components.mine
  if mine and mine.issprung then
    CancelDelayedReset(self)
    return DoReset(self)
  end
end
local OnTakeFuel
OnTakeFuel = function(self)
  return TryStartDelayedReset(self)
end
local OnDropped
OnDropped = function(self)
  return self.components.mine:Deactivate()
end
local OnDeploy
OnDeploy = function(self, pt, deployer)
  DoReset(self)
  return self.Physics:Teleport(pt:Get())
end
local LongUpdate
LongUpdate = function(self, dt)
  if self.reset_targettime then
    return StartDelayedReset(self, (self.reset_targettime - GetTime() - dt))
  end
end
local OnSave
OnSave = function(self, data)
  if self.reset_targettime then
    data.reset_delay = math.max(0, (self.reset_targettime - GetTime()))
  end
  return nil
end
local OnLoad
OnLoad = function(self, data)
  if data and data.reset_delay then
    StartDelayedReset(self.self, data.reset_delay)
  end
  return nil
end
local ApplyOverrides
do
  local inventoryact_override
  inventoryact_override = function(self, doer, actions, right)
    local mine
    do
      local _obj_0 = self.inst
      mine = _obj_0.components.mine
    end
    if right or (mine and mine.inactive and not mine.issprung) then
      local ret
      self.canbepickedup = true
      local InventoryItem = require("components/inventoryitem")
      ret = InventoryItem.CollectSceneActions(self, doer, actions, false)
      self.canbepickedup = false
      return ret
    end
  end
  local canacceptfuel_override
  canacceptfuel_override = function(self, item)
    if not (self:GetPercent() > MYTUNING.FUEL_ACCEPTANCE_THRESHOLD) then
      return Fueled.CanAcceptFuelItem(self, item)
    end
  end
  ApplyOverrides = function(self)
    self.components.mine.CollectSceneActions = false
    self.components.inventoryitem.CollectSceneActions = inventoryact_override
    self.components.fueled.CanAcceptFuelItem = canacceptfuel_override
  end
end
local AddEventListeners
do
  local raw_startreset
  raw_startreset = function(self)
    return StartDelayedReset(self)
  end
  AddEventListeners = function(self)
    self:ListenForEvent("onputininventory", CancelDelayedReset)
    self:ListenForEvent("enterlimbo", CancelDelayedReset)
    self:ListenForEvent("exitlimbo", TryStartDelayedReset)
    return self:ListenForEvent("ontrigger", raw_startreset)
  end
end
local fn
fn = function()
  local inst = CreateEntity()
  do
    local _with_0 = inst
    _with_0.entity:AddTransform()
    _with_0.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    do
      local _with_1 = inst.entity:AddMiniMapEntity()
      _with_1:SetIcon("toothtrap.png")
    end
    do
      local _with_1 = inst.entity:AddAnimState()
      _with_1:SetBank("trap_teeth")
      _with_1:SetBuild("clockwork_trap_teeth")
      _with_1:PlayAnimation("idle")
    end
    _with_0:AddTag("trap")
    _with_0:AddComponent("inspectable")
    _with_0:AddComponent("inventoryitem")
    do
      local _with_1 = inst.components.inventoryitem
      _with_1.nobounce = true
      _with_1.canbepickedup = false
      _with_1:SetOnDroppedFn(OnDropped)
      _with_1.atlasname = "images/inventoryimages/clockwork_trap_teeth.xml"
    end
    _with_0:AddComponent("mine")
    do
      local _with_1 = inst.components.mine
      _with_1:SetRadius(MYTUNING.RADIUS)
      _with_1:SetAlignment("player")
      _with_1:SetOnExplodeFn(OnExplode)
      _with_1:SetOnResetFn(OnReset)
      _with_1:SetOnSprungFn(SetSprung)
      _with_1:SetOnDeactivateFn(SetInactive)
    end
    _with_0:AddComponent("fueled")
    do
      local _with_1 = inst.components.fueled
      _with_1.maxfuel = MYTUNING.USES
      _with_1:InitializeFuelLevel(MYTUNING.USES)
      _with_1.fueltype = "TOOTH"
      _with_1.ontakefuelfn = OnTakeFuel
      _with_1.accepting = true
    end
    _with_0:AddComponent("deployable")
    do
      local _with_1 = inst.components.deployable
      _with_1.ondeploy = OnDeploy
      _with_1.min_spacing = .75
    end
    ApplyOverrides(inst)
    AddEventListeners(inst)
    _with_0.components.mine:Deactivate()
    _with_0.OnLongUpdate = LongUpdate
    _with_0.OnLoad = OnLoad
    _with_0.OnSave = OnSave
    return _with_0
  end
end
return unpack({
  Prefab("common/inventory/clockwork_trap_teeth", fn, assets),
  MakePlacer("common/clockwork_trap_teeth_placer", "trap_teeth", "clockwork_trap_teeth", "idle")
})
