MYTUNING = TUNING.CLOCKWORK_TRAP_TEETH

Fueled = require "components/fueled"


assets = {
	Asset "ANIM", "anim/clockwork_trap_teeth.zip"
}


OnExplode = (target) =>
	@AnimState\PlayAnimation "trap"
	if target
		@SoundEmitter\PlaySound "dontstarve/common/trap_teeth_trigger"
		target.components.combat\GetAttacked self, MYTUNING.DAMAGE
	if @components.fueled
		@components.fueled\DoDelta -1
	@PushEvent "ontrigger", {:target}

SetSprung = =>
	@AnimState\PlayAnimation "trap_idle"

SetInactive = =>
	@AnimState\PlayAnimation "inactive"

CanReset = =>
	{components: {:fueled}} = self
	not (fueled and fueled\IsEmpty!)

DoReset = =>
	{components: {:mine}} = self
	if @IsValid! and mine
		if CanReset self
			mine\Reset!
		else
			mine.inactive = false
			mine.issprung = true
			SetSprung self

CancelDelayedReset = =>
	if @task then
		@task\Cancel!
		@task = nil
	@reset_targettime = nil

StartDelayedReset = (delay) =>
	CancelDelayedReset self

	if CanReset self
		delay = math.max 0, (delay or MYTUNING.RESET_DELAY)
		@reset_targettime = GetTime! + delay
		@task = @DoTaskInTime delay, DoReset

OnReset = =>
	@SoundEmitter\PlaySound "dontstarve/common/trap_teeth_reset"
	@AnimState\PlayAnimation "reset"
	@AnimState\PushAnimation "idle", false
	CancelDelayedReset self

TryStartDelayedReset = =>
	{components: {:mine}} = self
	if mine and mine.issprung
		CancelDelayedReset self
		DoReset self

OnTakeFuel = =>
	TryStartDelayedReset self

OnDropped = =>
	@components.mine\Deactivate!

OnDeploy = (pt, deployer) =>
	DoReset self
	@Physics\Teleport pt\Get!

LongUpdate = (dt) =>
	StartDelayedReset self, (@reset_targettime - GetTime! - dt) if @reset_targettime

OnSave = (data) =>
	data.reset_delay = math.max 0, (@reset_targettime - GetTime!) if @reset_targettime
	nil

OnLoad = (data) =>
	StartDelayedReset @self, data.reset_delay if data and data.reset_delay
	nil

ApplyOverrides = do
	inventoryact_override = (doer, actions, right) =>
		{components: {:mine}} = @inst
		if right or (mine and mine.inactive and not mine.issprung)
			local ret
			@canbepickedup = true
			InventoryItem = require "components/inventoryitem"
			ret = InventoryItem.CollectSceneActions self, doer, actions, false
			@canbepickedup = false
			ret

	canacceptfuel_override = (item) =>
		Fueled.CanAcceptFuelItem self, item unless @GetPercent! > MYTUNING.FUEL_ACCEPTANCE_THRESHOLD

	=>
		@components.mine.CollectSceneActions = false
		@components.inventoryitem.CollectSceneActions = inventoryact_override
		@components.fueled.CanAcceptFuelItem = canacceptfuel_override

AddEventListeners = do
	raw_startreset = =>
		StartDelayedReset self

	=>
		@ListenForEvent "onputininventory", CancelDelayedReset
		@ListenForEvent "enterlimbo", CancelDelayedReset
		@ListenForEvent "exitlimbo", TryStartDelayedReset
		@ListenForEvent "ontrigger", raw_startreset


fn = ->
	inst = CreateEntity!
	with inst
		--[[
		-- Engine-level components.
		--]]

		.entity\AddTransform!
		.entity\AddSoundEmitter!
		MakeInventoryPhysics inst
		
		with inst.entity\AddMiniMapEntity!
			\SetIcon "toothtrap.png"
	   
		with inst.entity\AddAnimState!
			\SetBank "trap_teeth"
			\SetBuild "clockwork_trap_teeth"
			\PlayAnimation "idle"


		--[[
		-- Tags.
		--]]
		
		\AddTag "trap"


		--[[
		-- Components.
		--]]
		
		\AddComponent "inspectable"

		\AddComponent "inventoryitem"
		with inst.components.inventoryitem
			.nobounce = true
			.canbepickedup = false
			\SetOnDroppedFn OnDropped
			.atlasname = "images/inventoryimages/clockwork_trap_teeth.xml"
		
		\AddComponent "mine"
		with inst.components.mine
			\SetRadius MYTUNING.RADIUS
			\SetAlignment "player"
			\SetOnExplodeFn OnExplode
			\SetOnResetFn OnReset
			\SetOnSprungFn SetSprung
			\SetOnDeactivateFn SetInactive
		
		\AddComponent "fueled"
		with inst.components.fueled
			.maxfuel = MYTUNING.USES
			\InitializeFuelLevel MYTUNING.USES
			.fueltype = "TOOTH"
			--\SetDepletedFn OnDepleted
			.ontakefuelfn = OnTakeFuel
			.accepting = true
		
		\AddComponent "deployable"
		with inst.components.deployable
			.ondeploy = OnDeploy
			.min_spacing = .75


		--[[
		-- Further initialization.
		--]]
		
		ApplyOverrides inst
		AddEventListeners inst
		.components.mine\Deactivate!


		.OnLongUpdate = LongUpdate
		.OnLoad = OnLoad
		.OnSave = OnSave


unpack {
	Prefab "common/inventory/clockwork_trap_teeth", fn, assets
	MakePlacer "common/clockwork_trap_teeth_placer", "trap_teeth", "clockwork_trap_teeth", "idle"
}
